package ro.tremend.upload;

import com.missiondata.fileupload.MonitoredDiskFileItemFactory;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * This servlet is called only once when upload button is pressed. The actual progress requests are made
 * in UploadProxy.
 * UploadServlet starts monitoring the upload by setting some objects on the HTTP session so that
 * they'll be available to further periodic requests to UploadProxy. When finishing, it sets the status
 * of the upload on the session again, so that the very next progress request will know it has to stop.
 * <p/>
 * To ease the usage of this class as a decoupled module, unique identifiers can be associated to the
 * uploaded files. When finishing up, this servlet will make available on the session in a map, the FileItem
 * object under the key specified (the unique identifier passed)
 * <p/>
 * Originally from carsonm: http://blogs.missiondata.com/java/28/file-upload-progress-with-ajax-and-java-and-prototype/
 *
 * @author Marius Hanganu
 * @see ro.tremend.upload.UploadStatus
 */
public class UploadServlet extends HttpServlet {
    public static final String UPLOAD_STATUS = "uploadStatus";
    public static final String FILE_UPLOAD_STATS = "fileUploadStats";
    public static final String FILES_UPLOADED = "FILES_UPLOADED";
    public static final String UNIQUE_IDENTIFIER = "uniqueFileIdentifier";

    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        HttpSession session = request.getSession();

        UploadStatus status = new UploadStatus();
        session.setAttribute(UploadServlet.UPLOAD_STATUS, null);

        try {
            FileUploadListener listener = new FileUploadListener(request.getContentLength());
            session.setAttribute(FILE_UPLOAD_STATS, listener.getFileUploadStats());
            FileItemFactory factory = new MonitoredDiskFileItemFactory(listener);
            ServletFileUpload upload = new ServletFileUpload(factory);
            List items = upload.parseRequest(request);
            boolean hasError = false;

            // process the files
            storeFilesOnSession(request, items);

            // one can retrieve the FileItem objects in a different servlet, using
            // List filesUploaded = (List) session.getAttribute(UploadServlet.FILES_UPLOADED);

            if (!hasError) {
                status.setStatus(UploadStatus.STATUS_OK);
            } else {
                status.setStatus(UploadStatus.STATUS_ERROR);
                status.setMessage("Could not process uploaded file. Please see log for details.");
            }
        }
        catch (Exception e) {
            status.setStatus(UploadStatus.STATUS_ERROR);
            status.setMessage(e.getMessage());
        }
        session.setAttribute(UPLOAD_STATUS, status);
    }

    private void storeFilesOnSession(HttpServletRequest request, List items) {
        // check if there's a request to identify the file to be uploaded
        String uniqueIdentifier = request.getParameter(UNIQUE_IDENTIFIER);
        if (uniqueIdentifier != null) {
            // if an identifier for these files has been passed, make the FileItem objects available
            // in the session
            List<FileItem> files = (List<FileItem>) request.getSession().getAttribute(uniqueIdentifier);
            if (files == null) {
                files = new ArrayList<FileItem>();
            }

            for (Iterator i = items.iterator(); i.hasNext();) {
                FileItem fileItem = (FileItem) i.next();
                if (!fileItem.isFormField()) {
                    files.add(fileItem);
                }
            }
            request.getSession().setAttribute(FILES_UPLOADED, files);
        }
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        doPost(request, response);
    }
}
